@@ -4,7 +4,7 @@
 #    Project: Fast Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2017-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2017-2024 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -34,7 +34,7 @@ __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "29/08/2023"
+__date__ = "16/05/2024"
 __status__ = "production"
 
 import logging
@@ -394,6 +394,7 @@ def unBinning(*args, **kwargs):
     return unbinning(*args, **kwargs)
 
 
+
 def shift_fft(input_img, shift_val, method="fft"):
     """Do shift using FFTs
 
@@ -716,7 +717,7 @@ def roundfft(*args, **kwargs):
     return round_fft(*args, **kwargs)
 
 
-def is_far_from_group(pt, lst_pts, d2):
+def is_far_from_group_python(pt, lst_pts, d2):
     """
     Tells if a point is far from a group of points, distance greater than d2 (distance squared)
 
@@ -732,6 +733,12 @@ def is_far_from_group(pt, lst_pts, d2):
             return False
     return True
 
+try:
+    from ..ext.mathutil import is_far_from_group_cython
+except ImportError:
+    is_far_from_group = is_far_from_group_python
+else:
+    is_far_from_group = is_far_from_group_cython
 
 def rwp(obt, ref, scale=1.0):
     """Compute :math:`\\sqrt{\\sum \\frac{4\\cdot(obt-ref)^2}{(obt + ref)^2}}`.
@@ -922,3 +929,13 @@ def interp_filter(ary, out=None):
     out[mask_invalid] = numpy.interp(x[mask_invalid], x[mask_valid], ary[mask_valid],
                                      left=first, right=last)
     return out
+
+
+def allclose_mod(a, b, modulo=2*numpy.pi, **kwargs):
+    """Returns True if the two arrays a & b are equal within the given
+    tolerance modulo `modulo`; False otherwise.
+
+    Thanks to "Serguei Sokol" <sokol@insa-toulouse.fr>
+    """
+    di = numpy.minimum((a-b)%modulo, (b-a)%modulo)
+    return numpy.allclose(modulo*0.5, (di+modulo*0.5), **kwargs)
