@@ -4,7 +4,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2015-2023 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2015-2024 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -82,7 +82,7 @@ __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "16/11/2023"
+__date__ = "25/04/2024"
 __status__ = "development"
 
 import threading
@@ -90,7 +90,6 @@ import os.path
 import logging
 import json
 import numpy
-from collections import OrderedDict
 
 logger = logging.getLogger(__name__)
 
@@ -206,6 +205,8 @@ def _reduce_images(filenames, method="mean"):
     :param List[str] filenames: List of files used to compute the data
     :param str method: method used to compute the dark, "mean" or "median"
     """
+    if isinstance(filenames, str):
+        return pyFAI.io.image.read_image_data(filenames).astype(numpy.float32)
     if len(filenames) == 0:
         return None
     if len(filenames) == 1:
@@ -217,13 +218,13 @@ def _reduce_images(filenames, method="mean"):
 class Worker(object):
 
     def __init__(self, azimuthalIntegrator=None,
-                 shapeIn=(2048, 2048), shapeOut=(360, 500),
+                 shapeIn=None, shapeOut=(360, 500),
                  unit="r_mm", dummy=None, delta_dummy=None,
                  method=("bbox", "csr", "cython"),
                  integrator_name=None, extra_options=None):
         """
         :param AzimuthalIntegrator azimuthalIntegrator: An AzimuthalIntegrator instance
-        :param tuple shapeIn: image size in input
+        :param tuple shapeIn: image size in input ->auto guessed from detector shape now
         :param tuple shapeOut: Integrated size: can be (1,2000) for 1D integration
         :param str unit: can be "2th_deg, r_mm or q_nm^-1 ...
         :param float dummy: the value making invalid pixels
@@ -256,7 +257,7 @@ class Worker(object):
         self.extension = None
         self.needs_reset = True
         self.output = "numpy"  # exports as numpy array by default
-        self.shape = shapeIn
+        self._shape = shapeIn
         self.radial = None
         self.azimuthal = None
         self.radial_range = None
@@ -439,6 +440,12 @@ class Worker(object):
 
     setFlatfieldFile = set_flat_field_file
 
+    def set_mask_file(self, imagefile):
+        self.ai.set_mask(mask=_reduce_images(imagefile))
+        self.mask_image = imagefile
+
+    setMaskFile = set_mask_file
+
     def set_config(self, config, consume_keys=False):
         """
         Configure the working from the dictionary.
@@ -707,6 +714,16 @@ class Worker(object):
             raise_exception(reason)
         return reason
 
+    @property
+    def shape(self):
+        try:
+            shape = self.ai.detector.shape
+        except Exception as err:
+            logger.warning("The detector does not define its shape !")
+            return self._shape
+        else:
+            return shape
+
 
 class PixelwiseWorker(object):
     """
